@@ -154,10 +154,11 @@ def jax_funcify_Scan(op: Scan, **kwargs):
             for init_state, trace, buffer in zip(init_states, traces, buffers):
                 if init_state is not None:
                     # MIT-SOT and SIT-SOT: The final output should be as long as the input buffer
-                    full_trace = jnp.concatenate(
-                        [jnp.atleast_1d(init_state), jnp.atleast_1d(trace)],
-                        axis=0,
+                    trace = jnp.atleast_1d(trace)
+                    init_state = jnp.expand_dims(
+                        init_state, range(trace.ndim - init_state.ndim)
                     )
+                    full_trace = jnp.concatenate([init_state, trace], axis=0)
                     buffer_size = buffer.shape[0]
                 else:
                     # NIT-SOT: Buffer is just the number of entries that should be returned
